/*-+------------+-------+--------    +-------------------------------------------------------------
   |            |       |            |
   |            |       |            |
   |            |       |            |
   |            |       |            |
---+------------+-------+------------+-------------------------------------------------------------*/
/* mros2 example
 * Copyright (c) 2021 smorita_emb
 */
 
#include "mbed.h"
#include "mbed_wait_api.h"
#include "mros2.h"
#include "mros2-platform.h"
#include "std_msgs/msg/u_int64.hpp"
#include "mbed2rasppi_msgs/msg/mbed2rasppi.hpp"
#include "rasppi2mbed_msgs/msg/rasppi2mbed.hpp"
#include "rtos.h" 
//#include "TextLCD.h"

//--------------------入出力ピン定義---------------------
//青ボタン設定
DigitalIn btn1(PC_13);
//LEDの設定
DigitalOut led1(LED1);
DigitalOut led2(LED2);
DigitalOut led3(LED3);
//LCDの設定
//TextLCD lcd(PG_1, PE_3, PE_6, PE_5, PE_4, PE_2);//(RS,E,DB4,DB5,DB6,DB7)
//---------------------変数定義
uint8_t v_bluestat_rpi1 = 0;
float v_bluestat_rpi2 = 0;
uint8_t v_bluestat1 = 0;
float v_bluestat2 = 0;
uint8_t t10ms_flag = 0;
uint8_t t20ms_flag = 0;
uint8_t t50ms_flag = 0;
uint8_t t100ms_flag = 0;
uint8_t t500ms_flag = 0;

//---------------------トピック通信割り込み関数定義--------------------
mros2::Publisher pub;
mros2::Subscriber sub;
//---------------------トピック通信送信データ変数定義-- V4.0.0.2 引っ越し------------------
//mbed2rasppi_msgs::msg::Mbed2rasppi pubmsg;
//---------------------タイマ割り込み関数定義--------------------
Ticker t10msec;
Ticker t20msec;
Ticker t50msec;
Ticker t100msec;
Ticker t500msec;
//---------------------初期化処理宣言--------------------------------
int initialize();//・・・後方に記載
//-------------------タイマ割込みタスク 宣言--------------------
void t10task();
void t20task();
void t50task();
void t100task();
void t500task();
//------------------------- 以下関数定義-------------------------------

//送信定義
mbed2rasppi_msgs::msg::Mbed2rasppi pubmsg;
//送信時実行タスク
int mros_send(){
    //message param set
    pubmsg.v_bluestat1 = v_bluestat1;
    pubmsg.v_bluestat2 = v_bluestat2;
    //messae send
    pub.publish(pubmsg);
    return 0;
}

//受信割り込み発生時タスク
void rasppi2mbed_subscription(rasppi2mbed_msgs::msg::Rasppi2mbed *msg){

    v_bluestat_rpi1 = msg->v_bluestat_rpi1;//受信信号をRAMに放り込む
    v_bluestat_rpi2 = msg->v_bluestat_rpi2;//受信信号をRAMに放り込む

    MROS2_INFO("subscripting custom msg!!");
} 


int t500main(){
    // 500msecタスク処理
           t500ms_flag = 0;
           led3 = !led3;//STEP4
           return 0;
}
int t100main(){//---------------------100msec 処理系---------------------
    t100ms_flag = 0;
    return 0;
}
    	
int t50main(){//STEP4
    t50ms_flag = 0;
    return 0;//STEP4
} 

int t20main(){
    t20ms_flag = 0;
	return 0;
} 

int t10main(){//----------------------10msec 処理系---------------------
    t10ms_flag = 0;

    if(btn1 != 0){
        v_bluestat1 = 1;
        v_bluestat2 = 0;
    }
    else{
        v_bluestat1 = 0;
        v_bluestat2 = 1;
    }

    led1 = v_bluestat_rpi1;
    led2 = int8_t(v_bluestat_rpi2);


    return 0;
}
//----------------------メイン-------------------------	
int main() {
           initialize();

           printf("mbed mros2 start!\r\n");
           printf("app name: MBED2RaspPi ticker_test\r\n");
           mros2_platform::network_connect();

           mros2::init(0, NULL);
           MROS2_DEBUG("mROS 2 initialization is completed\r\n");

           mros2::Node node = mros2::Node::create_node("mbed2rasppi_custom_msg");
           MROS2_INFO("ready to node\r\n");

           pub = node.create_publisher<mbed2rasppi_msgs::msg::Mbed2rasppi>("mbed2rasppi", 10);
           osDelay(100);
           MROS2_INFO("ready to pub message\r\n");

           sub = node.create_subscription<rasppi2mbed_msgs::msg::Rasppi2mbed>("rasppi2mbed", 10, rasppi2mbed_subscription);
           osDelay(100);
           MROS2_INFO("ready to sub message\r\n");
           
           MROS2_INFO("ready to custom message\r\n");

        //--------定時タスク処理開始---------
           t10msec.attach(&t10task, 10ms);    // 10msec
           t20msec.attach(&t20task, 20ms);    // 20msec
           t50msec.attach(&t50task, 50ms);    // 50msec
           t100msec.attach(&t100task, 100ms);    // 100msec
           t500msec.attach(&t500task, 500ms);    // 500msec
 
           while(1)
           { 
               if(t10ms_flag==1){     // 10msecタスク処理
                   t10main();
               }
               if(t20ms_flag==1){     // 20msecタスク処理
                   t20main();
               }
               if(t50ms_flag==1){     // 50msecタスク処理
                   t50main();
               }
               if(t100ms_flag==1){     // 100msecタスク処理
                   mros_send();
                   t100main();
               }
               if(t500ms_flag==1){     // 500msecタスク処理
                   t500main();
               }
               else{}
               //ThisThread::sleep_for(1);
               wait_us(100);
               // wait(0.01);
               // osDelay(100);
           }
               
           mros2::spin();
           return 0;
}
	
//-------------------タイマ割込みタスク実体定義--------------------
//厳密な時間管理が必要な指示のみここに記載。
//オーバーフロー回避のためにも大体でよいものはメインループの中で実行すること		
void t10task(){ //STEP4
    t10ms_flag = 1; //10msec割り込みがかかったことをmain関数に知らせるフラグをセット
}

void t20task(){ //STEP4
    t20ms_flag = 1; //20msec割り込みがかかったことをmain関数に知らせるフラグをセット
}

void t50task(){ //STEP4
    t50ms_flag = 1; //50msec割り込みがかかったことをmain関数に知らせるフラグをセット
}
void t100task(){ //STEP4
    t100ms_flag = 1; //100msec割り込みがかかったことをmain関数に知らせるフラグをセット
}

void t500task(){
    t500ms_flag = 1; //500msec割り込みがかかったことをmain関数に知らせるフラグをセット
}

int initialize(){

    //LCD Initalize
    //lcd.init();
    //lcd.display(TextLCD::DISPLAY_ON);
    //lcd.cls();

    //初期画面
    //lcd.cls();
    //lcd.printf("MCU_%s",PROG_VER);//バージョン情報表示
    //lcd.locate(0,1);
    //lcd.printf("SBC_%s",BUDDY_SBC);
    //lcd.locate(20,0);
    //lcd.printf("mROS2%s starting",MROS2);

    //タイマー割り込み開始
    t500msec.attach(&t500task, 500ms);
    t100msec.attach(&t100task, 100ms);
    t50msec.attach(&t50task, 50ms);
    t20msec.attach(&t20task, 20ms);
    t10msec.attach(&t10task, 10ms);

    return 0;
}

